 
require "Window"

local DungeonGuinnessBook = {} -- see PublicEventStats

local savedVarsVersion = "sv_ver_3"

local ktRewardTierInfo = { -- TODO do we still need this?
	[PublicEvent.PublicEventRewardTier_None] 	= {strText = "None", strSprite = "CRB_ChallengeTrackerSprites:sprChallengeTierRed"}, -- 0
	[PublicEvent.PublicEventRewardTier_Bronze] 	= {strText = Apollo.GetString("ChallengeTier1"), strSprite = "CRB_ChallengeTrackerSprites:sprChallengeTierBronze"}, -- 1
	[PublicEvent.PublicEventRewardTier_Silver] 	= {strText = Apollo.GetString("ChallengeTier2"), strSprite = "CRB_ChallengeTrackerSprites:sprChallengeTierSilver"}, -- 2
	[PublicEvent.PublicEventRewardTier_Gold] 	= {strText = Apollo.GetString("ChallengeTier3"), strSprite = "CRB_ChallengeTrackerSprites:sprChallengeTierGold"}, -- 3
}

function DungeonGuinnessBook:new(o)
    o = o or {}
    setmetatable(o, self)
    self.__index = self 

	if(self.storedVars == nil) then
		self.storedVars = {}
	end
	
	-- init stuff if needed
	if(self.storedVars["settings"] == nil) then -- this should be in init or something but wont work then ... perhaps because saved vars getting restored..
		self.storedVars["settings"] = {
			notify_grp = true,
			show_record_timer = true, -- wether the record timer shall show up on dungeon enter
			max_last_runs = 0
		}
	end

    return o
end

function DungeonGuinnessBook:Init()
	local bHasConfigureFunction = false
	local strConfigureButtonText = ""
	local tDependencies = {
		-- "UnitOrPackageName",
	}
    Apollo.RegisterAddon(self, bHasConfigureFunction, strConfigureButtonText, tDependencies)

	Apollo.RegisterEventHandler("PublicEventStart","OnPublicEventStart", self) -- this is only for the timer window
	Apollo.RegisterEventHandler("PublicEventEnd","OnPublicEventEnd", self)
	
	Apollo.RegisterSlashCommand("dgb", "OnDungeonGuinnessBookOn", self)
	
	Apollo.RegisterEventHandler("WindowManagementReady", "OnWindowManagementReady", self)
	
	Apollo.RegisterEventHandler("AddonEvent_MenuEntryClicked", "OnMenuEntryClick", self)
	
	-- Timer
	Apollo.RegisterTimerHandler("close_char_record_notification", "DoCloseCharRecordNotifications", self)
	Apollo.RegisterTimerHandler("close_global_record_notification", "DoCloseGlobalRecordNotifications", self)
	Apollo.RegisterTimerHandler("update_rec_timer", "UpdateTimer", self)
	Apollo.RegisterTimerHandler("farm_summary", "DoShowFarmSummary", self)
	
	-- Menu item
	Apollo.RegisterEventHandler("InterfaceMenuListHasLoaded", "OnInterfaceMenuListHasLoaded", self)
	
	sv = savedVarsVersion -- shorten var access
	
	-- lang stuff
	if(Apollo.GetString(1) == "Abbrechen") then
		self.lang = Apollo.GetPackage("DGBLangDE").tPackage
		self.time_format = "%d.%m.%Y - %H:%M"
	elseif (Apollo.GetString(1) == "Cancel") then
		self.lang = Apollo.GetPackage("DGBLangEN").tPackage
		self.time_format = "%m/%d/%Y - %H:%M"
	elseif(Apollo.GetString(1) == "Annuler") then
		self.lang = Apollo.GetPackage("DGBLangFR").tPackage
	end
	
	-- !!!
	Apollo.RegisterSlashCommand("dtest", "test", self) -- remove later
	
	if(SendVarToRover == nil) then -- remove later
		SendVarToRover = function ()
		end
		
		rover = function ()
		end
	else
		rover = SendVarToRover
	end
	
	-- Style selection
	style = "DungeonGuinnessBook_Josch.xml"
	--style = "DungeonGuinnessBook_Nyo.xml"

end

-----------------------------------------------------------------------------------------------
-----------------------------------------------------------------------------------------------
-----------------------------------------------------------------------------------------------
function DungeonGuinnessBook:OnLoad()	
	self.wndMain = Apollo.LoadForm(style, "MainForm", nil, self)
    self.wndMain:Show(false)

	self.lastRunWin = Apollo.LoadForm(style, "LastRunForm", nil, self)
	self.lastRunWin:Show(false)
end

function DungeonGuinnessBook:test(cmd, param)
	--[[local ap = Apollo.GetPackage("autoprop").tPackage

	local foo = ap.create()
	
	local quxx = "testing"
	
	foo.a[quxx].c = "test"
	SendVarToRover("foo", foo)
	]]--
	--SendVarToRover("test2", foo.x.y.z)
	--SendVarToRover("test3", foo.a.b.c)
	
	local foo = {
		bar = "foobar",
		quxx = "quxxbar"
	}
	rover("foo", foo)
	--table.remove(foo,"bar")
	foo["bar"] = nil
	rover("foo2", foo)
end

function DungeonGuinnessBook:OnPublicEventStart(peStarting)

	if(self.timerWindow ~= nil) then -- how ever there are more then one pes started... so only accept the first one
		return
	end

	if (peStarting:GetEventType() ~= PublicEvent.PublicEventType_Dungeon and peStarting:GetEventType() ~= PublicEvent.PublicEventType_Shiphand) then
		return
	end
	
	--Print("PUBLIC EVENT STARTED")
	--SendVarToRover("peStarting", peStarting)
	
	local inst_name = peStarting:GetName()
	--local inst_name = GameLib.GetCurrentZoneMap()["strName"]
	local inst_cat = "cat_"..peStarting:GetEventType()
	local player_name = GameLib.GetPlayerUnit():GetName()
	local group_count = peStarting:GetTeamCount()
	
	if(self.storedVars[sv] ~= nil) then
		if(self.storedVars[sv]["dungeons"] ~= nil) then
			if(self.storedVars[sv]["dungeons"][inst_cat] ~= nil) then
				if(self.storedVars[sv]["dungeons"][inst_cat][inst_name] ~= nil) then
					if(self.storedVars[sv]["dungeons"][inst_cat][inst_name][player_name] ~= nil) then
						if(self.storedVars[sv]["dungeons"][inst_cat][inst_name][player_name][group_count] ~= nil) then
			
							self.timerWindow = Apollo.LoadForm(style, "RecordTimer", nil, self)
							
							local p = self.timerWindow:FindChild("ProgressBarChar")
							local pGlob = self.timerWindow:FindChild("ProgressBarGlobal")
							--local pPlayer = self.timerWindow:FindChild("ProgressBarXPlayers")
							
							p:SetMax(self.storedVars[sv]["dungeons"][inst_cat][inst_name][player_name][group_count]["time"])
							--pGlob:SetMax(self.storedVars[sv]["dungeons"]["GLOBAL"]["time"])
							pGlob:SetMax(self.storedVars[sv]["dungeons"][inst_cat][inst_name]["GLOBAL"]["time"])
							
							Apollo.CreateTimer("update_rec_timer", 1.0, true)
							
						end
					end
				end
			end
		end
	end
	
	self.peStartCurrencies = {
		gold 		= GameLib.GetPlayerCurrency():GetAmount(), 
		elder_gems 	= GameLib.GetPlayerCurrency(Money.CodeEnumCurrencyType.ElderGems):GetAmount(),
		renown 		= GameLib.GetPlayerCurrency(Money.CodeEnumCurrencyType.Renown):GetAmount(),
		glory 		= GameLib.GetPlayerCurrency(Money.CodeEnumCurrencyType.Glory):GetAmount(),
		omni_bits	= AccountItemLib.GetAccountCurrency(6):GetAmount(),
	}
end -- of peStart

function DungeonGuinnessBook:OnPublicEventEnd(peEnding, eReason, tStats)

	SendVarToRover("EndingType", peEnding:GetEventType())
	
	local isAdventure = (	   peEnding:GetEventType() == 15 
							or peEnding:GetEventType() == 17 
							or peEnding:GetEventType() == 14
							or peEnding:GetEventType() == 8
							or peEnding:GetEventType() == 18
							or peEnding:GetEventType() == 7
							or peEnding:GetEventType() == 13
							or peEnding:GetEventType() == 9
							or peEnding:GetEventType() == 12
						) -- this really sucks...

	if (peEnding:GetEventType() ~= PublicEvent.PublicEventType_Dungeon and peEnding:GetEventType() ~= PublicEvent.PublicEventType_Shiphand and isAdventure == false) then
		return -- we dont have any dungeon type -> do not log anything
	end

	SendVarToRover("peEnding", peEnding)
	--SendVarToRover("eReason", eReason)
	SendVarToRover("tStats", tStats)
	
	local inst_name = peEnding:GetName()
	--local inst_name = GameLib.GetCurrentZoneMap()["strName"]
	local inst_cat = "cat_"..peEnding:GetEventType()
	local player_name = GameLib.GetPlayerUnit():GetName()
	local realm = GameLib.GetRealmName()
	local group_count = table.getn( tStats.arParticipantStats )

	-- make sure the main storage structure is set
	if(self.storedVars == nil) then
		self.storedVars = {}
	end	
	if(self.storedVars[sv] == nil) then
		self.storedVars[sv] = {}
	end	
	if(self.storedVars[sv]["dungeons"] == nil) then
		self.storedVars[sv]["dungeons"] = {}
	end	
	if(self.storedVars[sv]["dungeons"][inst_cat] == nil) then
		self.storedVars[sv]["dungeons"][inst_cat] = {}
	end	
	if(self.storedVars[sv]["dungeons"][inst_cat][inst_name] == nil) then
		self.storedVars[sv]["dungeons"][inst_cat][inst_name] = {}
	end
	if(self.storedVars[sv]["dungeons"][inst_cat][inst_name][player_name] == nil) then
		self.storedVars[sv]["dungeons"][inst_cat][inst_name][player_name] = {}
	end
	if(self.storedVars[sv]["dungeons"][inst_cat][inst_name][player_name][group_count] == nil) then
		self.storedVars[sv]["dungeons"][inst_cat][inst_name][player_name][group_count] = {}
	end
	if(self.storedVars[sv]["dungeons"][inst_cat][inst_name][player_name][group_count]["medals"] == nil) then 
		self.storedVars[sv]["dungeons"][inst_cat][inst_name][player_name][group_count]["medals"] = {}
	end
	if(self.storedVars[sv]["dungeons"][inst_cat][inst_name][player_name][group_count]["roles"] == nil) then
		self.storedVars[sv]["dungeons"][inst_cat][inst_name][player_name][group_count]["roles"] = {}
	end
	if(self.storedVars[sv]["dungeons"][inst_cat][inst_name][player_name][group_count]["objectives"] == nil) then
		self.storedVars[sv]["dungeons"][inst_cat][inst_name][player_name][group_count]["objectives"] = {}
	end
	if(self.storedVars[sv]["dungeons"][inst_cat][inst_name]["GLOBAL"] == nil) then
		self.storedVars[sv]["dungeons"][inst_cat][inst_name]["GLOBAL"] = {}
	end
	if(self.storedVars[sv]["dungeons"][inst_cat][inst_name]["GLOBAL"]["times_avg"] == nil) then
		self.storedVars[sv]["dungeons"][inst_cat][inst_name]["GLOBAL"]["times_avg"] = {}
	end
	if(self.storedVars[sv]["dungeons"][inst_cat][inst_name]["GLOBAL"][group_count] == nil) then
		self.storedVars[sv]["dungeons"][inst_cat][inst_name]["GLOBAL"][group_count] = {}
	end
	if(self.storedVars[sv]["dungeons"][inst_cat][inst_name]["GLOBAL"][group_count]["medals"] == nil) then
		self.storedVars[sv]["dungeons"][inst_cat][inst_name]["GLOBAL"][group_count]["medals"] = {}
	end	
	
	if(self.storedVars[sv]["last_runs"] == nil) then
		self.storedVars[sv]["last_runs"] = {}
	end	
	
	-- 
	-- this is for easier debugging and may be removed later
	--self.storedVars["last_peEnding"] = peEnding -- this does not work because peEnding is type userdata which cannot be stored plain in savedVars
	self.storedVars["last_tStats"] = tStats
	--

	-- reference for var shortening
	local _glRecs = self.storedVars[sv]["dungeons"][inst_cat][inst_name]["GLOBAL"] -- we should also be able to write with _glRecs["time"] = foo
	
	--
	-- set GLOBAL record if the current run was better
	local force_record = false
	if(_glRecs["time"] == nil) then -- initialize if none-existing
		_glRecs["time"] = -1
		force_record = true -- this causes a record even if the recorded value is much bigger then the one to compare to
	end
	if(tStats.nElapsedTime < _glRecs["time"] or force_record) then -- global best time (group count independent) -> "just and only the over all best time"
		_glRecs["time"] = tStats.nElapsedTime
		_glRecs["timestamp"] =  os.time()
		_glRecs["char"] = player_name
		--Print(self.lang.created_global_record)
	end

	force_record = false
	if(_glRecs[group_count]["time"] == nil) then 
		_glRecs[group_count]["time"] = -1
		force_record = true
	end
	if(tStats.nElapsedTime < _glRecs[group_count]["time"] or force_record) then -- global best time (group strength dependent)
		_glRecs[group_count]["time"] = tStats.nElapsedTime
		_glRecs[group_count]["timestamp"] = os.time()
		_glRecs[group_count]["char"] = player_name
		
		--Print("You created a new global record for the current dungeon with a group count of "..group_count)
		self.globRecWin = Apollo.LoadForm(style, "GlobalRecordNotificationForm", nil, self)
		self.globRecWin:Show(true)
		Apollo.CreateTimer("close_global_record_notification", 6, false)
	end
	
	-- save time anyway for average run time
	table.insert(_glRecs["times_avg"], tStats.nElapsedTime)

	-- runs ..
	if(_glRecs["runs"] ~= nil) then
		_glRecs["runs"] = _glRecs["runs"] +1
	else
		_glRecs["runs"] = 1
	end
	
	if(_glRecs["kills"] ~= nil) then
		_glRecs["kills"] = _glRecs["kills"] + tStats["arPersonalStats"]["nKills"]
	else
		_glRecs["kills"] = tStats["arPersonalStats"]["nKills"]
	end
	
	if(tStats.eRewardType ~= PublicEvent.PublicEventRewardType_None) then -- prevent writing medal data if the dungeon does not even use medal system
		if(_glRecs[group_count]["medals"][tStats.eRewardTier] ~= nil) then
			_glRecs[group_count]["medals"][tStats.eRewardTier] = _glRecs[group_count]["medals"][tStats.eRewardTier] + 1
		else
			_glRecs[group_count]["medals"][tStats.eRewardTier] = 1
		end
	else
		_glRecs[group_count]["medals"]["no_medals"] = 1
	end
	
	-- shorten var access..
	local _deRecs = self.storedVars[sv]["dungeons"][inst_cat][inst_name][player_name] -- char dedicated records shorthanded
	
	--
	-- CHAR DEDICATED record
	force_record = false 
	if(_deRecs[group_count]["time"] == nil) then -- create if none-exist...
		_deRecs[group_count]["time"] = -1
		force_record = true
	end
	if(tStats.nElapsedTime < _deRecs[group_count]["time"] or force_record) then
		_deRecs[group_count]["time"] = tStats.nElapsedTime
		_deRecs[group_count]["timestamp"] = os.time() 

		-- UI Feedback for the record		
		self.charRecWin = Apollo.LoadForm(style, "CharRecordNotificationForm", nil, self)
		self.charRecWin:Show(true)
		Apollo.CreateTimer("close_char_record_notification", 6, false)
		if(self.storedVars["settings"]["notify_grp"]) then
			ChatSystemLib.Command("/i [DungeonGuinnessBook (beta)]: You helped me breaking a record for "..group_count.." players! We did it in "..DungeonGuinnessBook:HelperConvertTimeToString(tStats.nElapsedTime).."! Thank you!")
		end
	end

	-- runs
	if(_deRecs[group_count]["runs"] ~= nil) then
		_deRecs[group_count]["runs"] = _deRecs[group_count]["runs"] + 1
	else
		_deRecs[group_count]["runs"] = 1
	end
	
	-- best medal/reward tier
	if(tStats.eRewardType ~= PublicEvent.PublicEventRewardType_None) then
		if(_deRecs[group_count]["medals"][tStats.eRewardTier] ~= nil) then
			_deRecs[group_count]["medals"][tStats.eRewardTier] = _deRecs[group_count]["medals"][tStats.eRewardTier] + 1
		else
			_deRecs[group_count]["medals"][tStats.eRewardTier] = 1
		end
	else
		_deRecs[group_count]["medals"]["no_medals"] = 1
	end
	
	-- role and placing in dmg, heal and survival
	local player_role_str = "meta"
	if(GroupLib.InGroup()) then
		local player_role = GroupLib.GetGroupMember(1) -- the player is always the first party member
		if player_role.bTank then
			player_role_str = "tank" --Apollo.GetString("Group_TankTag")
		elseif player_role.bHealer then
			player_role_str = "healer" --Apollo.GetString("Group_HealerTag")
		elseif player_role.bDPS then
			player_role_str = "dd" --Apollo.GetString("Group_DPSTag")
		end
	end
		
	local rankings = self:Rank(tStats) -- nDamage, nDeaths, nHealed : "Count times beaten by other participants" -> taken from the original carbine addon
	local placing_dmg = rankings["nDamage"]
	local placing_heal = rankings["nHealed"]
	local placing_survive = rankings["nDeaths"]
	
	if(_deRecs[group_count]["roles"][player_role_str] == nil) then
		_deRecs[group_count]["roles"][player_role_str] = {
			placing_dmg = {}, -- these tables will contain how often you gained that placing 
			placing_heal = {},
			placing_survival = {}
		}
	end
	
	if(_deRecs[group_count]["roles"][player_role_str]["placing_dmg"]["rank_"..placing_dmg] ~= nil ) then
		_deRecs[group_count]["roles"][player_role_str]["placing_dmg"]["rank_"..placing_dmg] = _deRecs[group_count]["roles"][player_role_str]["placing_dmg"]["rank_"..placing_dmg] + 1
	else
		_deRecs[group_count]["roles"][player_role_str]["placing_dmg"]["rank_"..placing_dmg] = 1
	end
	
	if(_deRecs[group_count]["roles"][player_role_str]["placing_heal"]["rank_"..placing_heal] ~= nil ) then
		_deRecs[group_count]["roles"][player_role_str]["placing_heal"]["rank_"..placing_heal] = _deRecs[group_count]["roles"][player_role_str]["placing_heal"]["rank_"..placing_heal] + 1
	else
		_deRecs[group_count]["roles"][player_role_str]["placing_heal"]["rank_"..placing_heal] = 1
	end
	
	if(_deRecs[group_count]["roles"][player_role_str]["placing_survival"]["rank_"..placing_survive] ~= nil ) then
		_deRecs[group_count]["roles"][player_role_str]["placing_survival"]["rank_"..placing_survive] = _deRecs[group_count]["roles"][player_role_str]["placing_survival"]["rank_"..placing_survive] + 1
	else
		_deRecs[group_count]["roles"][player_role_str]["placing_survival"]["rank_"..placing_survive] = 1
	end
	
	-- objectives
	local id_type_to_string = {
		[PublicEventObjective.PublicEventStatus_Succeeded] = "success",
		[PublicEventObjective.PublicEventStatus_Failed] = "fail"
	}
	for k,v in pairs(tStats.arObjectives) do
		--Print(v.peoObjective:GetShortDescription() ..  "\n cat: " .. v.peoObjective:GetCategory() .. "\n objectiveType: " .. v.peoObjective:GetObjectiveType() .. "\n =========================")
		--if(v.peoObjective:GetCategory() == 3 and v.peoObjective:GetObjectiveType() == 5) then
		if(string.len(v.peoObjective:GetShortDescription()) > 1) then
			local descr = v.peoObjective:GetShortDescription()
			if( _deRecs[group_count]["objectives"][descr] == nil ) then
				_deRecs[group_count]["objectives"][descr] = { 
					success = 0,
					fail = 0
				}
			end
			
			if(v.eStatus == PublicEventObjective.PublicEventStatus_Succeeded or v.eStatus == PublicEventObjective.PublicEventStatus_Failed) then -- Other states include Succeeded, Active, Inactive and Failed
				local cur_status = id_type_to_string[v.eStatus]
				_deRecs[group_count]["objectives"][descr][cur_status] = _deRecs[group_count]["objectives"][descr][cur_status] + 1
			end
		end
	end
	
	--
	-- create last run entry
	self.storedVars[sv]["last_runs"][os.time()] = {
		time = tStats.nElapsedTime,
		char = player_name,
		role = player_role_str,
		group_strength = group_count,
		ranking = rankings,
		dungeon = inst_name,
		money_made = GameLib.GetPlayerCurrency():GetAmount() - self.peStartCurrencies["gold"],
		omni = AccountItemLib.GetAccountCurrency(6):GetAmount() - self.peStartCurrencies["omni_bits"],
		renown = GameLib.GetPlayerCurrency(Money.CodeEnumCurrencyType.Renown):GetAmount() - self.peStartCurrencies["renown"],
		elder_gems = GameLib.GetPlayerCurrency(Money.CodeEnumCurrencyType.ElderGems):GetAmount() - self.peStartCurrencies["elder_gems"],
		glory = GameLib.GetPlayerCurrency(Money.CodeEnumCurrencyType.Glory):GetAmount() - self.peStartCurrencies["glory"]
	}
	
	--
	-- show farm window with summary
	Apollo.CreateTimer("farm_summary", 1.5, false) -- why the timer? : After the event is finished you are granted some renown which will not be counted when not putting some time space between the summary and the instance end
end

function DungeonGuinnessBook:DoShowFarmSummary()
	self.farmWin = Apollo.LoadForm(style, "FarmSummaryForm", nil, self)
	
	local cc = self.farmWin:FindChild("CashContainer")
	cc:SetAmount(GameLib.GetPlayerCurrency():GetAmount() - self.peStartCurrencies["gold"])
	
	local table = self.farmWin:FindChild("Grid")
	
	local row = table:AddRow("")
	table:SetCellText(row, 1, "Elder Gems")
	table:SetCellText(row, 2, GameLib.GetPlayerCurrency(Money.CodeEnumCurrencyType.ElderGems):GetAmount() - self.peStartCurrencies["elder_gems"])
	
	local row = table:AddRow("")
	table:SetCellText(row, 1, "Renown")
	table:SetCellText(row, 2, GameLib.GetPlayerCurrency(Money.CodeEnumCurrencyType.Renown):GetAmount() - self.peStartCurrencies["renown"])

	local row = table:AddRow("")
	table:SetCellText(row, 1, "Omni")
	table:SetCellText(row, 2, AccountItemLib.GetAccountCurrency(6):GetAmount() - self.peStartCurrencies["omni_bits"])
	
	local row = table:AddRow("")
	table:SetCellText(row, 1, "Glory")
	table:SetCellText(row, 2, GameLib.GetPlayerCurrency(Money.CodeEnumCurrencyType.Glory):GetAmount() - self.peStartCurrencies["glory"])
	
	-- farming stats
	-- Gold: 			GameLib.GetPlayerCurrency():GetAmount() 
	
	-- ElderGems: 		GameLib.GetPlayerCurrency(Money.CodeEnumCurrencyType.ElderGems):GetAmount()
	-- Renown: 			GameLib.GetPlayerCurrency(Money.CodeEnumCurrencyType.Renown):GetAmount()
	-- Glory: 			GameLib.GetPlayerCurrency(Money.CodeEnumCurrencyType.Glory):GetAmount()
	
	-- OmniBits: 		AccountItemLib.GetAccountCurrency(6):GetAmount()
	
	-- self.peStartCurrencies
	--[[
		self.peStartCurrencies = {
		gold 		= GameLib.GetPlayerCurrency():GetAmount(), 
		elder_gems 	= GameLib.GetPlayerCurrency(Money.CodeEnumCurrencyType.ElderGems):GetAmount()
		renown 		= GameLib.GetPlayerCurrency(Money.CodeEnumCurrencyType.Renown):GetAmount()
		glory 		= GameLib.GetPlayerCurrency(Money.CodeEnumCurrencyType.Glory):GetAmount()
		omni_bits	= AccountItemLib.GetAccountCurrency(6):GetAmount()
	}
	]]--
end

function DungeonGuinnessBook:OnDungeonGuinnessBookOn(cmd, param, self_ref) -- slash command handler

	if(self_ref ~= nil) then -- needed when the ws menu to the left bottom is used
		self = self_ref
	end

	local list = self.wndMain:FindChild("Dungeons")
	
	local detailStage = self.wndMain:FindChild("DetailStage")
	detailStage:DestroyChildren()
	
	if(	self.storedVars[sv] ~= nil ) then -- when the book is opend before there was anything recorded 

					SendVarToRover("storedVars", self.storedVars)
	
		list:DestroyChildren()
		
		for cat, dungeons in pairs(self.storedVars[sv]["dungeons"]) do
			self:HelperAddDungeonToList(list, cat, nil, true, cat)
			for dName, dData in pairs(dungeons) do
				self:HelperAddDungeonToList(list, dName, dName, false, cat)
			end
		end
		
		list:ArrangeChildrenVert()
	
	else
		self:HelperAddDungeonToList(list, "Nothing recorded yet! Do a Dungeon!" , nil)
	end
	
	self.wndMain:Invoke() -- show the window
	
	self.wndMain:SetSizingMinimum(600, 400) -- prevent the window from being resized smaller then this
end

function DungeonGuinnessBook:HelperAddDungeonToList(list, caption, detailId, isCat, catId)
	if(isCat == nil) then isCat = false end
	local li = nil
	if(isCat) then
		local cat_id = string.sub(catId, 5) + 0
		if(cat_id == PublicEvent.PublicEventType_Dungeon) then
			caption = Apollo.GetString("ZoneStateDungeon")
		elseif (cat_id == PublicEvent.PublicEventType_Shiphand ) then
			caption = Apollo.GetString("MatchMaker_Shiphands")
		else
			caption = Apollo.GetString("MatchMaker_Adventures")
		end
		li = Apollo.LoadForm(style, "DungeonCat", list, self)
	else
		li = Apollo.LoadForm(style, "ListItem", list, self)
	end
	li:SetText(" "..caption)
	li:SetData({
		id = detailId,
		cat = catId
	})
end

function DungeonGuinnessBook:HelperAddLastRunEntryToList(list, caption, detailId)
	local li = Apollo.LoadForm(style, "ListItemLastRuns", list, self)

	li:SetText(" "..caption)
	li:SetData({
		id = detailId
	})
end

function DungeonGuinnessBook:HelperAddSegmentToDetail(stage, text)
	local li = Apollo.LoadForm(style, "DetailStageRow", stage, self)
	li:SetAML(text.."\n 0")  -- when the text is not followed by a new line and something behind the new line, the height adjustment does not work properly
	li:SetHeightToContentHeight() -- only works for windows of class MLWindow!
	return li -- so the fresh element could be used for something (for example as new parent for cascaded call)
end

function DungeonGuinnessBook:HelperAddCategorySegmentToDetail(stage, sText, sTitle)
	local form = Apollo.LoadForm(style, "DetailCategory", stage, self)
	
	local title = form:FindChild("CatTitleText")
	local content = form:FindChild("CatContent")
	
	title:SetText(sTitle)
	
	content:SetAML(sText.."\n")
	content:SetHeightToContentHeight()
	
	local cDims = {} -- content dims
	cDims[0], cDims[1], cDims[2], cDims[3] = content:GetAnchorOffsets()
	
	local fDims = {} -- container form dims
	fDims[0], fDims[1], fDims[2], fDims[3] = form:GetAnchorOffsets()
	
	form:SetAnchorOffsets(fDims[0],fDims[1],fDims[2],cDims[3]+fDims[3]-62) 
	
	return form, content
end

function DungeonGuinnessBook:HelperAddHeadlineToDetail(stage, text, master)
	local dsDims = {} -- container frame dims
	dsDims[0],dsDims[1],dsDims[2],dsDims[3] = stage:GetAnchorOffsets()

	if(master == nil) then master = false end
	
	local hl = nil
	if (master == true) then
		hl = Apollo.LoadForm(style, "DetailMasterHeadlineRow", stage, self)
	else
		text = "  "..text
		hl = Apollo.LoadForm(style, "DetailNormalHeadlineRow", stage, self)
	end
	--hl:SetSprite("BK3:sprMetal_ExpandMenu_Large_Framing") -- Anim_Inventory_New:sprInventory_NewItemScale
	
	local dims = {} -- current segment dims
	dims[0],dims[1],dims[2],dims[3] = hl:GetAnchorOffsets(0)
	
	hl:SetText(text)
	
	--hl:SetAML("<P Font=\"CRB_HeaderGigantic\" Align=\"Right\">"..text.."</P>")
		
				--hl:SetAnchorOffsets(dims[0],dims[1],(dsDims[2]-dsDims[3])*2-20, dims[3]+14) -- wtf calc. I really don't know why but this works perfectly
				
	--hl:SetTextFlags("DT_RIGHT", true) -- not avail to MLWindow elems
end

--
-- Delegator function: For elements are reused this is needed to decide which handler to use........
function DungeonGuinnessBook:DungeonButtonClicked( wndHandler, wndControl, eMouseButton, nLastRelativeMouseX, nLastRelativeMouseY )
	local detail_func = wndControl:GetFrame():GetName()
	
	if(detail_func == "MainForm") then
		DungeonGuinnessBook:ShowDungeonDetail(self, wndHandler, wndControl)
	elseif (detail_func == "LastRunForm") then
		DungeonGuinnessBook:ShowLastRunDetail(self, wndHandler, wndControl)
	end
end

--
-- SHOW LAST RUN DETAIL
function DungeonGuinnessBook:ShowLastRunDetail(self, wndHandler, wndControl)
	local detailStage = self.lastRunWin:FindChild("LRContent")	
	detailStage:DestroyChildren()
	
	-- add spacer to the top
	Apollo.LoadForm(style, "Spacer", detailStage, self)
	
	local id = wndControl:GetData()["id"]
	
	DungeonGuinnessBook:HelperAddHeadlineToDetail(detailStage, self.storedVars[sv]["last_runs"][id]["dungeon"])
	
	-- Run detail
	local catForm, catContent = DungeonGuinnessBook:HelperAddCategorySegmentToDetail(detailStage, "" , "Run detail")
	
	DungeonGuinnessBook:HelperAddSegmentToDetail(catContent, "Character: " .. self.storedVars[sv]["last_runs"][id]["char"])
	DungeonGuinnessBook:HelperAddSegmentToDetail(catContent, "Time: " .. DungeonGuinnessBook:HelperConvertTimeToString(self.storedVars[sv]["last_runs"][id]["time"]).." minutes")
	DungeonGuinnessBook:HelperAddSegmentToDetail(catContent, "Role: " .. self.storedVars[sv]["last_runs"][id]["role"])
	DungeonGuinnessBook:HelperAddSegmentToDetail(catContent, "Groupmember count: " .. self.storedVars[sv]["last_runs"][id]["group_strength"])
	if(self.storedVars[sv]["last_runs"][id]["group_strength"] > 1) then
		DungeonGuinnessBook:HelperAddSegmentToDetail(catContent, 
													"Ranking (dmg): " .. self.storedVars[sv]["last_runs"][id]["ranking"]["nDamage"] .. 
													"\nRanking (healing): " .. self.storedVars[sv]["last_runs"][id]["ranking"]["nHealed"] .. 
													"\nRanking (survive): " .. self.storedVars[sv]["last_runs"][id]["ranking"]["nDeaths"])
	end
	
	catContent:ArrangeChildrenVert()
	DungeonGuinnessBook:HelperAdjustContentHeightAfterwards(catContent)
	DungeonGuinnessBook:HelperAdjustContentHeightAfterwards(catForm)
	
	-- Run detail
	local catForm, catContent = DungeonGuinnessBook:HelperAddCategorySegmentToDetail(detailStage, "" , "Farm detail")
	
	DungeonGuinnessBook:HelperAddSegmentToDetail(catContent, "Glory: " .. self.storedVars[sv]["last_runs"][id]["glory"])
	DungeonGuinnessBook:HelperAddSegmentToDetail(catContent, "Renown: " .. self.storedVars[sv]["last_runs"][id]["renown"])
	DungeonGuinnessBook:HelperAddSegmentToDetail(catContent, "Omni: " .. self.storedVars[sv]["last_runs"][id]["omni"])
	DungeonGuinnessBook:HelperAddSegmentToDetail(catContent, "Elder Gems: " .. self.storedVars[sv]["last_runs"][id]["elder_gems"])
	
	catContent:ArrangeChildrenVert()
	DungeonGuinnessBook:HelperAdjustContentHeightAfterwards(catContent)
	DungeonGuinnessBook:HelperAdjustContentHeightAfterwards(catForm)
	
	-- Add Money made
	local cc = self.lastRunWin:FindChild("CashContainer")
	cc:SetAmount(self.storedVars[sv]["last_runs"][id]["money_made"])
	
	detailStage:ArrangeChildrenVert()
end

-- SHOW CATEGORY DETAIL
function DungeonGuinnessBook:ShowCategoryDetail(self, wndHandler, wndControl)
	local detailStage = self.wndMain:FindChild("DetailStage")	
	detailStage:DestroyChildren()
	
	local cat_id = string.sub(wndControl:GetData()["cat"], 5) + 0 -- c&p work
	if(cat_id == PublicEvent.PublicEventType_Dungeon) then
		caption = Apollo.GetString("ZoneStateDungeon")
	elseif (cat_id == PublicEvent.PublicEventType_Shiphand ) then
		caption = Apollo.GetString("MatchMaker_Shiphands")
	else
		caption = Apollo.GetString("MatchMaker_Adventures")
	end
	
	DungeonGuinnessBook:HelperAddHeadlineToDetail(detailStage, caption)
	
	local player_name = GameLib.GetPlayerUnit():GetName()
	local dungeons = self.storedVars[sv]["dungeons"][wndControl:GetData()["cat"]]
	
	for k,v in pairs(dungeons) do
		local catForm, catContent = DungeonGuinnessBook:HelperAddCategorySegmentToDetail(detailStage, "", k)
		
		DungeonGuinnessBook:HelperAddSegmentToDetail(catContent, 
			"<T TextColor=\"ChatTrade\">Best time by:</T> \n" .. 
			v["GLOBAL"]["char"] .. " <T TextColor=\"ChatTrade\">in</T> " .. os.date("%H:%M", v["GLOBAL"]["time"]) .. " minutes <T TextColor=\"ChatTrade\">at</T> " .. 
			os.date(self.time_format, v["GLOBAL"]["timestamp"])
		)
		
		DungeonGuinnessBook:HelperAddSegmentToDetail(catContent, "<T TextColor=\"ChatTrade\">Runs:</T> " .. v["GLOBAL"]["runs"])
		DungeonGuinnessBook:HelperAddSegmentToDetail(catContent, "<T TextColor=\"ChatTrade\">Kills:</T> " .. v["GLOBAL"]["kills"])
		
		catContent:ArrangeChildrenVert()
		DungeonGuinnessBook:HelperAdjustContentHeightAfterwards(catContent)
		DungeonGuinnessBook:HelperAdjustContentHeightAfterwards(catForm)
	end
	detailStage:ArrangeChildrenVert()
end

function DungeonGuinnessBook:HelperAdjustContentHeightAfterwards(stage)
	local neededHeight = 0
	local si = 0 -- saved i
	for i, c in pairs(stage:GetChildren()) do
		neededHeight = neededHeight + c:GetHeight()
		si = i
	end
	local dims = {}
	dims[0],dims[1],dims[2],dims[3] = stage:GetAnchorOffsets() -- left, top, right, bottom
	stage:SetAnchorOffsets(dims[0],dims[1],dims[2], neededHeight+si*4)
end

--
-- SHOW DUNGEON DETAIL
function DungeonGuinnessBook:ShowDungeonDetail(self, wndHandler, wndControl)

	-- UI feedback
	for k, obj in pairs(self.wndMain:FindChild("Dungeons"):GetChildren()) do -- reset colors to standard
		obj:SetTextColor("gray")
		obj:SetTextColor("gray")
	end
	
	wndControl:SetBGColor("white")
	wndControl:SetTextColor("white")
	
	--
	-- show detail
	local detailStage = self.wndMain:FindChild("DetailStage")
	detailStage:DestroyChildren()

	local dungeon2show = wndControl:GetData()["id"]
	local dungeonCat = wndControl:GetData()["cat"]
	
	-- error message if needed
	if(dungeon2show == nil) then
		DungeonGuinnessBook:HelperAddSegmentToDetail(detailStage, "You did not do any dungeons while this addon was installed. Do one to make it start some recording!")
		return
	end
	
	local player_name = GameLib.GetPlayerUnit():GetName()
	
	local global_recs_group = self.storedVars[sv]["dungeons"][dungeonCat][dungeon2show]["GLOBAL"] -- pointer to the global records which also have group count -> use global_recs_group[1/2/3...]

	-- detail headline: dungeon name
	DungeonGuinnessBook:HelperAddHeadlineToDetail(detailStage, dungeon2show, true)

	-- first row: char and global record
	DungeonGuinnessBook:HelperAddSegmentToDetail(detailStage, 
		replaceUnicodeEntities(self.lang.best_time_overall) .. 
		"<T TextColor=\"ChatTrade\">" .. DungeonGuinnessBook:HelperConvertTimeToString(global_recs_group["time"]) .. "</T>" .. 
		" "..self.lang.minutes_by.." <T TextColor=\"ChatTrade\">" .. global_recs_group["char"] .. "</T>" .. 
		" "..self.lang.on.." <T TextColor=\"ChatTrade\">" .. os.date(self.time_format, global_recs_group["timestamp"]).."\n</T>"
	)
	
	if(self.storedVars[sv]["dungeons"][dungeonCat][dungeon2show][player_name] ~= nil) then
		-- dungeon runs / count
		local group_runs_string = ""
		for k,v in pairs(self.storedVars[sv]["dungeons"][dungeonCat][dungeon2show][player_name]) do
			if(k==1) then
				group_runs_string = "\n * Solo runs: " .. v["runs"]
			else
				group_runs_string = group_runs_string .. "\n * Group of " .. k .. " Players: " .. v["runs"]
			end
			--group_runs_string = group_runs_string .. " / " .. v["kills"] .. " kills" -- there is no kills for the record with a group count
		end
		DungeonGuinnessBook:HelperAddSegmentToDetail(detailStage,
			"<T TextColor=\"ChatTrade\" Font=\"Thick\">Runs:</T> \nAccountwide: " .. global_recs_group["runs"].."\n"..
			player_name..": " .. group_runs_string
		)
		
		-- iterate over group counts and print global and char specific data
		for k,v in pairs(self.storedVars[sv]["dungeons"][dungeonCat][dungeon2show][player_name]) do -- char dedicated
		
			local caption = nil
			if(k==1) then
				caption = "Solo"
			else
				caption = k.." Player Record"
			end
			DungeonGuinnessBook:HelperAddHeadlineToDetail(detailStage, caption)

			DungeonGuinnessBook:HelperAddSegmentToDetail(detailStage, 
				"<T TextColor=\"ChatTrade\">Current Character:</T> " .. DungeonGuinnessBook:HelperConvertTimeToString(v["time"]) .. " minutes on " .. os.date(self.time_format, v["timestamp"]) .. "\n" .. 
				"<T TextColor=\"ChatTrade\">Account:</T> " .. DungeonGuinnessBook:HelperConvertTimeToString(global_recs_group[k]["time"]) .. " minutes on " .. os.date(self.time_format, global_recs_group[k]["timestamp"])
			)
			
			-- medals
			--local medals_string = "<T TextColor=\"ChatTrade\" Font=\"Thick\">Medals:</T> \n"
			local medals_string = ""
			if(v["medals"]["no_medals"] == nil) then
				for rewardTierId,timesGained in pairs(v["medals"]) do
					 medals_string = medals_string .. ktRewardTierInfo[rewardTierId]["strText"]..": "..timesGained.." times\n"
				end
			else
				medals_string = medals_string .. "This dungeon does not use medal system! You can not be granted any."
			end
			--DungeonGuinnessBook:HelperAddSegmentToDetail(detailStage, medals_string)
			DungeonGuinnessBook:HelperAddCategorySegmentToDetail(detailStage, medals_string, "Medals")
			
			-- objectives
			local i = 0
			for objDescr, objStatRatio in pairs(v["objectives"]) do -- there is no other way... table.getn(.. does not work for assoc tables
				i = i +1
				break
			end
			if(i > 0) then
				--DungeonGuinnessBook:HelperAddSegmentToDetail(detailStage, "<T TextColor=\"ChatTrade\">Objectives:</T> \n")
				local contentArea = nil
				local catForm = nil -- form, content
				catForm, contentArea = DungeonGuinnessBook:HelperAddCategorySegmentToDetail(detailStage, "", "Objectives")
				--local table = Apollo.LoadForm(style, "TableListContainer", detailStage, self)
				local table = Apollo.LoadForm(style, "TableListContainer", contentArea, self)
				for objDescr, objStatRatio in pairs(v["objectives"]) do
					 --objectives_string = objectives_string .. objDescr..": "..objStatRatio["success"].." (successed) / "..objStatRatio["fail"].." (failed)\n"
					local row = table:AddRow("")
					table:SetCellText(row, 1, objDescr)
					table:SetCellText(row, 2, objStatRatio["success"])
					table:SetCellText(row, 3, objStatRatio["fail"])
				end
				local rows = table:GetRowCount() -- for calc
				
				-- adjust table height
				local dims = {}
				dims[0],dims[1],dims[2],dims[3] = table:GetAnchorOffsets() -- left, top, right, bottom
				table:SetAnchorOffsets(dims[0],dims[1],dims[2], 35+rows*15+5)
								
				-- adjust category form height
				catForm:SetAnchorOffsets(dims[0],dims[1],dims[2],35+rows*15+80)				
				
			else
				--DungeonGuinnessBook:HelperAddSegmentToDetail(detailStage, "<T TextColor=\"ChatTrade\">Objectives:</T> \nThis dungeon does not have ending criteria objectives") 
				DungeonGuinnessBook:HelperAddCategorySegmentToDetail(detailStage, "This dungeon does not have ending criteria objectives", "Objectives")
			end
			
			-- roles
			--local roles_string = "<T TextColor=\"ChatTrade\">Roles:</T> \n"
			local roles_string = ""
			for role, placingData in pairs(v["roles"]) do
				if(role ~= "meta") then -- prevent stats and role for solo runs ... bacause this would not make any sense
					roles_string = roles_string .. role .. ":\n"
					for placing_string, arRanks in pairs(placingData) do
						local role_type = string.sub(placing_string, 9)
						roles_string = roles_string .. " - placing in " .. role_type .. ":\n"
						for ranking_string, val in pairs(arRanks) do
							local rank = string.sub(ranking_string, 6) + 1
							ranking_string = "<T TextColor=\"ChatTrade\">"..rank .. ". place: </T>"
							roles_string = roles_string .. "    " ..ranking_string .. " " .. val .. " time(s)\n"
						end
					end
				else
					roles_string = roles_string .. "Roles and rankings for solo runs are dispensable\n"
				end
			end			
			--DungeonGuinnessBook:HelperAddSegmentToDetail(detailStage, roles_string)
			DungeonGuinnessBook:HelperAddCategorySegmentToDetail(detailStage, roles_string, "Roles")

		end
		
	end
	
	detailStage:ArrangeChildrenVert()
end

--
-- obligatory stuff
--
function DungeonGuinnessBook:OnCancel()
	self.wndMain:Close()
end

function DungeonGuinnessBook:OnWindowManagementReady() -- persists the window state
	Event_FireGenericEvent("WindowManagementRegister", {strName = "DungeonGuinnessBookMain", nSaveVersion=1}) 
	Event_FireGenericEvent("WindowManagementRegister", {strName = "DungeonGuinnessBookLastRun", nSaveVersion=1})
	
    Event_FireGenericEvent("WindowManagementAdd", {wnd = self.wndMain, strName = "DungeonGuinnessBookMain", nSaveVersion=1})
	Event_FireGenericEvent("WindowManagementAdd", {wnd = self.lastRunWin, strName = "DungeonGuinnessBookLastRun", nSaveVersion=1})
end

function DungeonGuinnessBook:OnSave(eType)
	--GameLib.CodeEnumAddonSaveLevel.General
	if(eType == GameLib.CodeEnumAddonSaveLevel.Account) then
		return self.storedVars 
	end
end

function DungeonGuinnessBook:OnRestore(eType, tSavedData)
	if(eType == GameLib.CodeEnumAddonSaveLevel.Account) then
		self.storedVars = tSavedData
	end
end

function DungeonGuinnessBook:OnMenuEntryClick()
	DungeonGuinnessBook:OnDungeonGuinnessBookOn(nil,nil, self)
end

function DungeonGuinnessBook:OnInterfaceMenuListHasLoaded()
	Event_FireGenericEvent("InterfaceMenuList_NewAddOn", "Dungeon Guinness Book", {"AddonEvent_MenuEntryClicked", "", "DungeonGuinnessBookSprites:DGBIcon"})
end

--
-- "lib" stuff
--
function round(num, idp)
	local mult = 10^(idp or 0)
	return math.floor(num * mult + 0.5) / mult
end

function replaceUnicodeEntities(str)
	local replacements = {
	   	[""] = "\195\132",
		[""] = "\195\150",
		[""] = "\195\156",
		[""] = "\195\159",
   		[""] = "\195\164",
		[""] = "\195\182",
		[""] = "\195\188"
	}
	for find, replace in pairs(replacements) do
		str = string.gsub(str, find, replace)
	end	
	return str
end

function DungeonGuinnessBook:Rank(tStats)

	local tPersonalStats = tStats.arPersonalStats
	local participant_stats = tStats.arParticipantStats
	
	local tScore = {["nDamage"] = 0, ["nHealed"] = 0, ["nDeaths"] = 0}

	-- Add Custom to score tracker
	for idx, tStat in pairs(tPersonalStats.arCustomStats) do -- tPersonalStats.arCustomStats
		if tStat.nValue and tStat.nValue > 0 then
			tScore[tStat.strName] = 0
			tPersonalStats[tStat.strName] = tStat.nValue
		end
	end

	-- Count times beaten by other participants
	for key, tCurr in pairs(participant_stats) do -- self.tZombieStats.arParticipantStats
		tScore = self:HelperCompareAdventureScores(tPersonalStats, tCurr, tScore)
	end
	
	return tScore
end

function DungeonGuinnessBook:HelperCompareAdventureScores(tPersonalStats, tComparisonStats, tScore)
	if tComparisonStats.nDeaths < tPersonalStats.nDeaths then
		tScore.nDeaths = tScore.nDeaths + 1
	end
	if tComparisonStats.nDamage > tPersonalStats.nDamage then
		tScore.nDamage = tScore.nDamage + 1
	end
	if tComparisonStats.nHealed > tPersonalStats.nHealed then
		tScore.nHealed = tScore.nHealed + 1
	end

	for idx, tCompareStat in pairs(tComparisonStats.arCustomStats) do
		local tPersonalStat = tPersonalStats.arCustomStats[idx]
		if tPersonalStat and tCompareStat.nValue > tPersonalStat.nValue then
			if tScore[tCompareStat.strName] then
				tScore[tCompareStat.strName] = tScore[tCompareStat.strName] + 1
			else
				tScore[tCompareStat.strName] = 1
			end
		end
	end

	return tScore
end

function DungeonGuinnessBook:HelperConvertTimeToString(fTime)
	fTime = math.floor(fTime / 1000) -- TODO convert to full seconds

	return string.format("%d:%02d", math.floor(fTime / 60), math.floor(fTime % 60))
end

function DungeonGuinnessBook:DoCloseCharRecordNotifications() -- timer functions
	self.charRecWin:Destroy()
end

function DungeonGuinnessBook:DoCloseGlobalRecordNotifications() 
	self.globRecWin:Destroy()
end

function DungeonGuinnessBook:UpdateTimer()	
	if(PublicEvent.GetActiveEvents()[1] ~= nil) then
	
		local p = self.timerWindow:FindChild("ProgressBarChar")
		local pGlob = self.timerWindow:FindChild("ProgressBarGlobal")
	
		local elapsed_time = PublicEvent.GetActiveEvents()[1]:GetElapsedTime()
		
		if( p:GetMax() >= elapsed_time ) then
			p:SetProgress(elapsed_time)
		else
			Print("Failed in setting a new record for this dungeon with your current character. Sorry :(")
		end
		
		if( pGlob:GetMax() >= elapsed_time) then
			pGlob:SetProgress(elapsed_time)
		else
			Print("Failed in setting a new accountwide record for this dungeon. Sorry :(")
		end
		
		if( pGlob:GetMax() < elapsed_time and p:GetMax() < elapsed_time ) then
			Apollo.CreateTimer("update_rec_timer", 1, false)  -- make the timer not recurring; this is a hack cuz just calling StopTimer simply does not work
			Apollo.StopTimer("update_rec_timer") 
			Print("Right now you ran out of all time for setting any new record for this dungeons :(")
		end
		
	else
		Apollo.CreateTimer("update_rec_timer", 1, false)
		Apollo.StopTimer("update_rec_timer") -- this does NOT stop the timer... 
	end
	
	-- TODO: the timer window should be destroyed on zone change or something. Otherwise if a new instance is started a old timer will not be closeable any more (without reloadui)
	-- or use a timer which closes the frame after 30 secons
	-- or close the old timer before one new can be spawned
end

---------------------------------------------------------------------------------------------------
-- Form Functions
---------------------------------------------------------------------------------------------------

function DungeonGuinnessBook:ShowSettings( wndHandler, wndControl, eMouseButton )
	if(self.cfgForm == nil) then
		self.cfgForm = Apollo.LoadForm(style, "SettingsForm", nil, self)
		self.cfgForm:FindChild("set_notify_group_record"):SetCheck(self.storedVars["settings"]["notify_grp"])
	end
end

function DungeonGuinnessBook:OnCbNotifyGroupRecordCheck( wndHandler, wndControl, eMouseButton )
	self.storedVars["settings"]["notify_grp"] = true
end

function DungeonGuinnessBook:OnCbNotifyGroupRecordUncheck( wndHandler, wndControl, eMouseButton )
	self.storedVars["settings"]["notify_grp"] = false
end

function DungeonGuinnessBook:OnConfigCloseBtn( wndHandler, wndControl, eMouseButton )
	self.cfgForm:Destroy()
	self.cfgForm = nil
end

function DungeonGuinnessBook:OnShowLastRunBtn( wndHandler, wndControl, eMouseButton )
	self.lastRunWin:Show(true)
	self.lastRunWin:ToFront()
	self.lastRunWin:SetSizingMinimum(600, 400)
	
	local list = self.lastRunWin:FindChild("LRList")
	list:DestroyChildren()
	
	-- add spacer to the top
	Apollo.LoadForm(style, "Spacer", list, self)	
	
	if(self.storedVars[sv] ~= nil) then
		if(self.storedVars[sv]["last_runs"] ~= nil) then
			for date,data in pairs(self.storedVars[sv]["last_runs"]) do
				local forename = string.match(data["char"], "(.*)%s")
				--self:HelperAddDungeonToList(list, os.date(self.time_format, date) .. "\n" .. forename .. "\n" .. data["dungeon"], date)
				self:HelperAddLastRunEntryToList(list, os.date(self.time_format, date) .. "\n " .. forename .. "\n " .. data["dungeon"], date)
			end
		end
	end
	list:ArrangeChildrenVert()
end

function DungeonGuinnessBook:OnCloseLastRunBtn( wndHandler, wndControl, eMouseButton )
	self.lastRunWin:Show(false)
end

function DungeonGuinnessBook:OnShowGlobalDungeonStatsBtn( wndHandler, wndControl, eMouseButton )
	Print("DungeonGuinnessBook Global Dungeon Stats: NOT YET IMPLEMENTED")
end

function DungeonGuinnessBook:OnFarmFromCloseBtn( wndHandler, wndControl, eMouseButton )
	self.farmWin:Destroy()
end

function DungeonGuinnessBook:OnTimerCloseBtn( wndHandler, wndControl, eMouseButton )
	Apollo.CreateTimer("update_rec_timer", 1, false)
	Apollo.StopTimer("update_rec_timer") 
	self.timerWindow:Destroy()
	self.timerWindow = nil
end

function DungeonGuinnessBook:OnFarmCloseBtn( wndHandler, wndControl, eMouseButton )
	self.farmWin:Destroy()
end

function DungeonGuinnessBook:OnCategoryButtonClick( wndHandler, wndControl, eMouseButton )
	DungeonGuinnessBook:ShowCategoryDetail(self, wndHandler, wndControl)
end

function DungeonGuinnessBook:OnDeleteLastRunButton( wndHandler, wndControl, eMouseButton )
	local id = wndControl:GetParent():GetData()["id"]
	self.storedVars[sv]["last_runs"][id] = nil
	self:OnShowLastRunBtn(wndHandler, wndControl)
end

-----------------------------------------------------------------------------------------------
-- DungeonGuinnessBook Instance
-----------------------------------------------------------------------------------------------
local DungeonGuinnessBookInst = DungeonGuinnessBook:new()
DungeonGuinnessBookInst:Init()
